# Generated by Django 2.2.10 on 2020-05-12 08:44
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("storage", "0002_create_filestorage_objects"),
    ]

    operations = [
        migrations.AddField(
            model_name="referencedpath",
            name="awss3etag",
            field=models.CharField(default="etag", max_length=50),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="referencedpath",
            name="crc32c",
            field=models.CharField(default="crc32c", max_length=8),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="referencedpath",
            name="md5",
            field=models.CharField(default="md5", max_length=32),
            preserve_default=False,
        ),
    ]
