# Generated by Django 2.2.9 on 2020-04-14 21:44
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):
    dependencies = [
        ("flow", "0043_full_text_search"),
    ]
    operations = [
        migrations.AlterField(
            model_name="data",
            name="location",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                to="storage.FileStorage",
                related_name="data",
            ),
        ),
        migrations.DeleteModel(
            name="DataLocation",
        ),
        migrations.AlterField(
            model_name="data",
            name="status",
            field=models.CharField(
                choices=[
                    ("UP", "Uploading"),
                    ("RE", "Resolving"),
                    ("WT", "Waiting"),
                    ("PP", "Preparing"),
                    ("PR", "Processing"),
                    ("OK", "Done"),
                    ("ER", "Error"),
                    ("DR", "Dirty"),
                ],
                default="RE",
                max_length=2,
            ),
        ),
    ]
